<?php
//
//	Module name      :	odbcperm.php
//	Version          :	1.0
//	Started on       :	April 26, 2007
//	Completed on     :	May 10, 2007
//
//	Module type      :	Library
//	Description      :	Functions to check permissions on forms, views and form controls, stored in a DB
//						accessed via Odbc module
//
//	Developed under  :	Php 4.1.2
//	Sw requirements  :	None
//
//	Parameters       :	None
//	Notes - usage    :	This module works on a database already initialized via Odbc layer and on tables
//						with the following minimal definitions:
//							CREATE TABLE User (
//								Id smallint(5) unsigned NOT NULL auto_increment,
//								GId smallint(5) unsigned NOT NULL default '0',
//								Name varchar(15) NOT NULL default '',
//								Disabled tinyint(1) unsigned NOT NULL default '0',
//								PRIMARY KEY  (Id),
//								UNIQUE KEY Name (Name)
//							);
//							CREATE TABLE Permission (
//								Id smallint(5) unsigned NOT NULL default '0',
//								GId smallint(5) unsigned NOT NULL default '0',
//								NameMask varchar(255) NOT NULL default '',
//								Form char(1) NOT NULL default '',
//								Views char(3) NOT NULL default '',
//								Controls1 varchar(10) NOT NULL default '',
//								Controls2 varchar(10) NOT NULL default '',
//								Controls3 varchar(10) NOT NULL default '',
//								PRIMARY KEY  (Id),
//								KEY GId (GId)
//							);
//						Table and field names can be changed using configuration descriptor in OPInit;
//						field lengths can be different (see descriptor).
//						This module can log performed queries using DbLog module and activating it
//						by $OP_db_log; queries are logged in a table named "Perm" (see $OP_log_table_name
//						to change its name), with definition:
//							CREATE TABLE Perm (
//								Ts int(11) NOT NULL default '0',
//								Ip varchar(15) default NULL,
//								Script varchar(24) default NULL,
//								ReqPath varchar(255) NOT NULL default '',
//								UserName varchar(15) NOT NULL default '',
//								PermId varchar(5) NOT NULL default '',
//								Perms varchar(perms_len, see formula below) NOT NULL default '',
//								KEY Ts (Ts),
//								KEY Ip (Ip)
//							);
//						Other indexes can be added; the field Perms will contain a string like (for default
//						values): "F:x;V:xxx;C1:xxxxxxxxxx;C2:xxxxxxxxxx;C3:xxxxxxxxxx;"; the length of this
//						field must be calculated with the formula: 
//							perms_len = (2 + 1 + 1) + (2 + $OP_desc["PermTable"]["ViewsNum"] + 1) +
//										(3 + $OP_desc["PermTable"]["ControlsNum"] + 1) *
//										$OP_desc["PermTable"]["ViewsNum"];
//						This module does not have error functions; error codes can be read calling Odbc
//						module functions.
//


//--- Include files --------------------------------------------------------------------------------



require_once("odbc.php");
require_once("dblog.php");



//--- Public constants -----------------------------------------------------------------------------



define("OP_MODULE_NAME", "odbcperm.php 1.0; on May 10, 2007 by J.Hubb.");



//--- Public variables -----------------------------------------------------------------------------



// Example for an uninitialized descriptor
$OP_desc = array(
		"UserTable"	=> array(
			"TableName"				=> "MyUser",
			"IdFieldName"			=> "Id",
			"GidFieldName"			=> "GId",
			"NameFieldName"			=> "Name",
			"DisabFieldName"		=> "Disabled"),
		"PermTable" => array(
			"TableName"				=> "MyPermission",
			"IdFieldName"			=> "Id",
			"GidFieldName"			=> "GId",
			"MaskFieldName"			=> "NameMask",
			"FormFieldName"			=> "Form",
			"ViewsFieldName"		=> "Views",
			"ViewsNum"				=> 3,
			"ControlsFieldPrefix"	=> "Controls",
			"ControlsNum"			=> 10)
		);

$OP_db_log = FALSE;				// Change to TRUE to Log queries by DBLog module
$OP_log_table_name = "Perm";	// Name of the table used by DBLog



//--- Private constants ----------------------------------------------------------------------------



//--- Private variables ----------------------------------------------------------------------------



$_OP_def_desc = array(
		"UserTable"	=> array(
			"TableName"				=> "User",
			"IdFieldName"			=> "Id",
			"GidFieldName"			=> "GId",
			"NameFieldName"			=> "Name",
			"DisabFieldName"		=> "Disabled"),
		"PermTable" => array(
			"TableName"				=> "Permission",
			"IdFieldName"			=> "Id",
			"GidFieldName"			=> "GId",
			"MaskFieldName"			=> "NameMask",
			"FormFieldName"			=> "Form",
			"ViewsFieldName"		=> "Views",
			"ViewsNum"				=> 3,
			"ControlsFieldPrefix"	=> "Controls",
			"ControlsNum"			=> 10)
		);



//--- Public functions code ------------------------------------------------------------------------



function OPControlPerm($cfg_desc, $control_num = 1, $view_num = 1)
// Returns the permission already read for the previously specified script, relative to the control
// $control_num of the view $view_num.
{
	if (($control_num > @$cfg_desc["PermTable"]["ControlsNum"]) ||
			($view_num > @$cfg_desc["PermTable"]["ViewsNum"]))
		return " ";
	else
		return @$cfg_desc["_ScriptPerm"]["Controls"][$view_num]{$control_num - 1};
}



function OPDone(&$desc)
// Closes the reference to specified database. Return nothing.
{
	unset($desc["_DbDesc"]);
	unset($desc["_ScriptPerm"]);
	return;
}



function OPInit(&$desc, &$db_desc)
// Initializes the descriptor $desc refererring to the database descriptor $db_desc and setting default
// values if not already set in $desc. Returns nothing.
{
	global $_OP_def_desc;

	$desc = array_merge($_OP_def_desc, $desc);		// Overwrite default values with new ones
	$desc["_DbDesc"] = &$db_desc;
	unset($desc["_ScriptPerm"]);
	return;
}



function OPFormPerm($cfg_desc)
// Returns the permission already read for the previously specified script, relative to the main form.
{
	return @$cfg_desc["_ScriptPerm"]["Form"];
}



function OPReadScriptPerm(&$cfg_desc, $path, $username)
// Returns permissions of the path $path, for the username $username. Returns FALSE on DB errors.
{
	global	$OP_db_log;

	$ctl_query = "p.{$cfg_desc["PermTable"]["ControlsFieldPrefix"]}1";
	for ($n = 2; $n <= $cfg_desc["PermTable"]["ViewsNum"]; $n++)
		$ctl_query .= ", p.{$cfg_desc["PermTable"]["ControlsFieldPrefix"]}$n";
	$tmp = ODBCSelectManyRows($cfg_desc["_DbDesc"],
			"SELECT p.{$cfg_desc["PermTable"]["IdFieldName"]}, " .
				"p.{$cfg_desc["PermTable"]["FormFieldName"]}, " .
				"p.{$cfg_desc["PermTable"]["ViewsFieldName"]}, " .
				"$ctl_query " .
			"FROM {$cfg_desc["PermTable"]["TableName"]} AS p " . 
			"LEFT JOIN {$cfg_desc["UserTable"]["TableName"]} AS u " .
			"ON p.{$cfg_desc["PermTable"]["GidFieldName"]} = u.{$cfg_desc["UserTable"]["GidFieldName"]} " .
			"WHERE (u.{$cfg_desc["UserTable"]["NameFieldName"]} = '$username') AND " .
				"(u.{$cfg_desc["UserTable"]["DisabFieldName"]} = 0) AND " .
				"('$path' LIKE p.{$cfg_desc["PermTable"]["MaskFieldName"]}) " .
			"ORDER BY p.{$cfg_desc["PermTable"]["IdFieldName"]}");
/*
	// To use regular expressions on file name masks:
			"WHERE (u.{$cfg_desc["UserTable"]["NameFieldName"]} = '$username') AND " .
				"('$path' REGEXP p.{$cfg_desc["PermTable"]["MaskFieldName"]}) " .
*/
	if ($OP_db_log)
	{
		$tmp2 = pathinfo(@$_SERVER["SCRIPT_NAME"]);
		if (count($tmp) == 0)
			DLAppend("Perm", @$_SERVER["REMOTE_ADDR"], $tmp2["basename"], $path, $username, "Empty", "");
		else
			foreach ($tmp as $perm)
			{
				$tmp_str = "F:{$perm[1]};V:{$perm[2]};";
				for ($n = 1; $n <= $cfg_desc["PermTable"]["ViewsNum"]; $n++)
					$tmp_str .= "C{$n}:{$perm[$n + 2]};";
				DLAppend("Perm", @$_SERVER["REMOTE_ADDR"], $tmp2["basename"], $path, $username, $perm[0],
						$tmp_str);
			}
	}
	$result = array(
			"Form"		=> str_pad(@$tmp[0][1], 1),			// If $tmp contains more rows, use only the first.
			"Views"		=> str_pad(@$tmp[0][2], $cfg_desc["PermTable"]["ViewsNum"]),
			"Controls"	=> array()
			);
	for ($n = 1; $n <= $cfg_desc["PermTable"]["ViewsNum"]; $n++)
		$result["Controls"][$n] = str_pad(@$tmp[0][$n + 2], $cfg_desc["PermTable"]["ControlsNum"]);
	$cfg_desc["_ScriptPerm"] = $result;
	return $result;
}



function OPViewPerm($cfg_desc, $view_num = 1)
// Returns the permission already read for the previously specified script, relative to the view $view_num.
{
	if ($view_num > @$cfg_desc["PermTable"]["ViewsNum"])
		return " ";
	else
		return @$cfg_desc["_ScriptPerm"]["Views"]{$view_num - 1};
}



//--- Private functions code -----------------------------------------------------------------------



//--- Initialization code --------------------------------------------------------------------------



?>
